@@ -26,9 +26,8 @@ del getLogger
 from gi.repository import Gtk, Gdk, GLib
 from gi.repository.GObject import TYPE_PYOBJECT
 
-
 from solaar import NAME
-from solaar.i18n import _
+from solaar.i18n import _, ngettext
 # from solaar import __version__ as VERSION
 from solaar.ui import ui_async as _ui_async
 from logitech_receiver import hidpp10 as _hidpp10
@@ -47,6 +46,12 @@ _NORMAL_BUTTON_ICON_SIZE = Gtk.IconSize.BUTTON
 _TREE_ICON_SIZE = Gtk.IconSize.BUTTON
 _INFO_ICON_SIZE = Gtk.IconSize.LARGE_TOOLBAR
 _DEVICE_ICON_SIZE = Gtk.IconSize.DND
+try:
+    import gi
+    gi.check_version("3.7.4")
+    _CAN_SET_ROW_NONE = None
+except (ValueError, AttributeError):
+    _CAN_SET_ROW_NONE = ''
 
 # tree model columns
 _COLUMN = _NamedInts(PATH=0, NUMBER=1, ACTIVE=2, NAME=3, ICON=4, STATUS_TEXT=5, STATUS_ICON=6, DEVICE=7)
@@ -55,30 +60,10 @@ _TREE_SEPATATOR = (None, 0, False, None, None, None, None, None)
 assert len(_TREE_SEPATATOR) == len(_COLUMN_TYPES)
 assert len(_COLUMN_TYPES) == len(_COLUMN)
 
-_TOOLTIP_LINK_SECURE = _("The wireless link between this device and its receiver is encrypted.")
-_TOOLTIP_LINK_INSECURE = _("The wireless link between this device and its receiver is not encrypted.\n"
-						"\n"
-						"For pointing devices (mice, trackballs, trackpads), this is a minor security issue.\n"
-						"\n"
-						"It is, however, a major security issue for text-input devices (keyboards, numpads),\n"
-						"because typed text can be sniffed inconspicuously by 3rd parties within range.")
-
-_UNIFYING_RECEIVER_TEXT = (
-		_("No device paired") + '.\n\n<small>' + _("Up to %d devices can be paired to this receiver") + '.</small>',
-		'%d ' + _("paired devices") + '\n\n<small>' + _("Up to %d devices can be paired to this receiver") + '.</small>',
-	)
-_NANO_RECEIVER_TEXT = (
-		_("No device paired") + '.\n\n<small> </small>',
-		' \n\n<small>' + _("Only one device can be paired to this receiver") + '.</small>',
-	)
-
 #
 # create UI layout
 #
 
-Gtk.Window.set_default_icon_name(NAME.lower())
-Gtk.Window.set_default_icon_from_file(_icons.icon_file(NAME.lower()))
-
 def _new_button(label, icon_name=None, icon_size=_NORMAL_BUTTON_ICON_SIZE, tooltip=None, toggle=False, clicked=None):
 	if toggle:
 		b = Gtk.ToggleButton()
@@ -306,9 +291,14 @@ def _create_window_layout():
 	assert _tree.get_selection().get_mode() == Gtk.SelectionMode.SINGLE
 	_tree.get_selection().connect('changed', _device_selected)
 
+	tree_scroll = Gtk.ScrolledWindow()
+	tree_scroll.add(_tree)
+	tree_scroll.set_min_content_width(_tree.get_size_request()[0])
+	tree_scroll.set_shadow_type(Gtk.ShadowType.IN)
+
 	tree_panel = Gtk.Box.new(Gtk.Orientation.VERTICAL, 0)
 	tree_panel.set_homogeneous(False)
-	tree_panel.pack_start(_tree, True, True, 0)
+	tree_panel.pack_start(tree_scroll, True, True, 0)
 	tree_panel.pack_start(_details, False, False, 0)
 
 	panel = Gtk.Box.new(Gtk.Orientation.HORIZONTAL, 16)
@@ -522,9 +512,9 @@ def _update_details(button):
 				yield (_("Index"), device.number)
 				yield (_("Wireless PID"), device.wpid)
 				hid_version = device.protocol
-				yield (_("Protocol"), 'HID++ %1.1f' % hid_version if hid_version else 'unknown')
+				yield (_("Protocol"), 'HID++ %1.1f' % hid_version if hid_version else _('Unknown'))
 				if read_all and device.polling_rate:
-					yield (_("Polling rate"), '%d ms (%dHz)' % (device.polling_rate, 1000 // device.polling_rate))
+					yield (_("Polling rate"), _('%(rate)d ms (%(rate_hz)dHz)') % { 'rate': device.polling_rate, 'rate_hz': 1000 // device.polling_rate })
 
 				if read_all or not device.online:
 					yield (_("Serial"), device.serial)
@@ -533,7 +523,7 @@ def _update_details(button):
 
 			if read_all:
 				for fw in list(device.firmware):
-					yield ('  ' +  str(fw.kind), (fw.name + ' ' + fw.version).strip())
+					yield ('  ' + _(str(fw.kind)), (fw.name + ' ' + fw.version).strip())
 			elif device.kind is None or device.online:
 				yield ('  %s' % _("Firmware"), '...')
 
@@ -575,16 +565,15 @@ def _update_receiver_panel(receiver, panel, buttons, full=False):
 	assert receiver
 
 	devices_count = len(receiver)
-	if receiver.max_devices > 1:
-		if devices_count == 0:
-			panel._count.set_markup(_UNIFYING_RECEIVER_TEXT[0] % receiver.max_devices)
-		else:
-			panel._count.set_markup(_UNIFYING_RECEIVER_TEXT[1] % (devices_count, receiver.max_devices))
-	else:
-		if devices_count == 0:
-			panel._count.set_markup(_NANO_RECEIVER_TEXT[0])
-		else:
-			panel._count.set_markup(_NANO_RECEIVER_TEXT[1])
+
+	paired_text = _('No device paired.') if devices_count == 0 else ngettext('%(count)s paired device.', '%(count)s paired devices.', devices_count) % { 'count': devices_count }
+
+	if(receiver.max_devices > 0):
+		paired_text += '\n\n<small>%s</small>' % ngettext('Up to %(max_count)s device can be paired to this receiver.', 'Up to %(max_count)s devices can be paired to this receiver.', receiver.max_devices) % { 'max_count': receiver.max_devices }
+	elif(devices_count > 0):
+		paired_text += '\n\n<small>%s</small>' % _('Only one device can be paired to this receiver.')
+
+	panel._count.set_markup(paired_text)
 
 	is_pairing = receiver.status.lock_open
 	if is_pairing:
@@ -630,9 +619,9 @@ def _update_device_panel(device, panel, buttons, full=False):
 		panel._battery._icon.set_sensitive(True)
 
 		if isinstance(battery_level, _NamedInt):
-			text = str(battery_level)
+			text = _(str(battery_level))
 		else:
-			text = '%d%%' % battery_level
+			text = _("%(battery_percent)d%%") % { 'battery_percent': battery_level }
 		if is_online:
 			if charging:
 				text += ' <small>(%s)</small>' % _("charging")
@@ -646,11 +635,16 @@ def _update_device_panel(device, panel, buttons, full=False):
 		if not_secure:
 			panel._secure._text.set_text(_("not encrypted"))
 			panel._secure._icon.set_from_icon_name('security-low', _INFO_ICON_SIZE)
-			panel._secure.set_tooltip_text(_TOOLTIP_LINK_INSECURE)
+			panel._secure.set_tooltip_text(_("The wireless link between this device and its receiver is not encrypted.\n"
+						"\n"
+						"For pointing devices (mice, trackballs, trackpads), this is a minor security issue.\n"
+						"\n"
+						"It is, however, a major security issue for text-input devices (keyboards, numpads),\n"
+						"because typed text can be sniffed inconspicuously by 3rd parties within range."))
 		else:
 			panel._secure._text.set_text(_("encrypted"))
 			panel._secure._icon.set_from_icon_name('security-high', _INFO_ICON_SIZE)
-			panel._secure.set_tooltip_text(_TOOLTIP_LINK_SECURE)
+			panel._secure.set_tooltip_text(_("The wireless link between this device and its receiver is encrypted."))
 		panel._secure._icon.set_visible(True)
 	else:
 		panel._secure._text.set_markup('<small>%s</small>' % _("offline"))
@@ -663,7 +657,7 @@ def _update_device_panel(device, panel, buttons, full=False):
 			panel._lux.set_visible(False)
 		else:
 			panel._lux._icon.set_from_icon_name(_icons.lux(light_level), _INFO_ICON_SIZE)
-			panel._lux._text.set_text('%d %s' % (light_level, _("lux")))
+			panel._lux._text.set_text(_("%(light_level)d lux") % { 'light_level': light_level })
 			panel._lux.set_visible(True)
 	else:
 		panel._lux.set_visible(False)
@@ -734,6 +728,9 @@ _window = None
 
 
 def init():
+	Gtk.Window.set_default_icon_name(NAME.lower())
+	Gtk.Window.set_default_icon_from_file(_icons.icon_file(NAME.lower()))
+
 	global _model, _tree, _details, _info, _empty, _window
 	_model = Gtk.TreeStore(*_COLUMN_TYPES)
 	_tree = _create_tree(_model)
@@ -777,7 +774,7 @@ def update(device, need_popup=False):
 		if is_alive and item:
 			was_pairing = bool(_model.get_value(item, _COLUMN.STATUS_ICON))
 			is_pairing = bool(device.status.lock_open)
-			_model.set_value(item, _COLUMN.STATUS_ICON, 'network-wireless' if is_pairing else None)
+			_model.set_value(item, _COLUMN.STATUS_ICON, 'network-wireless' if is_pairing else _CAN_SET_ROW_NONE)
 
 			if selected_device_id == (device.path, 0):
 				full_update = need_popup or was_pairing != is_pairing
@@ -803,13 +800,13 @@ def update(device, need_popup=False):
 
 			battery_level = device.status.get(_K.BATTERY_LEVEL)
 			if battery_level is None:
-				_model.set_value(item, _COLUMN.STATUS_TEXT, None)
-				_model.set_value(item, _COLUMN.STATUS_ICON, None)
+				_model.set_value(item, _COLUMN.STATUS_TEXT, _CAN_SET_ROW_NONE)
+				_model.set_value(item, _COLUMN.STATUS_ICON, _CAN_SET_ROW_NONE)
 			else:
 				if isinstance(battery_level, _NamedInt):
-					status_text = str(battery_level)
+					status_text = _("%(battery_level)s") % { 'battery_level': _(str(battery_level)) }
 				else:
-					status_text = '%d%%' % battery_level
+					status_text = _("%(battery_percent)d%%") % { 'battery_percent': battery_level }
 				_model.set_value(item, _COLUMN.STATUS_TEXT, status_text)
 
 				charging = device.status.get(_K.BATTERY_CHARGING)
